@@ -12,7 +12,7 @@
 int hc_lzma1_decompress (const unsigned char *in, SizeT *in_len, unsigned char *out, SizeT *out_len, const char *props);
 int hc_lzma2_decompress (const unsigned char *in, SizeT *in_len, unsigned char *out, SizeT *out_len, const char *props);
 
-void *hc_lzma_alloc (void *p, size_t size);
-void  hc_lzma_free  (void *p, void  *address);
+void *hc_lzma_alloc (MAYBE_UNUSED ISzAllocPtr p, size_t size);
+void  hc_lzma_free  (MAYBE_UNUSED ISzAllocPtr p, void *address);
 
 #endif // _EXT_LZMA_H
