@@ -7,6 +7,10 @@
 #include "types.h"
 #include "shared.h"
 
+#if defined (__CYGWIN__)
+#include <sys/cygwin.h>
+#endif
+
 static inline int get_msb32 (const u32 v)
 {
   int i;
@@ -151,28 +155,6 @@ void hc_asprintf (char **strp, const char *fmt, ...)
   va_end (args);
 }
 
-#if defined (_WIN)
-int hc_stat (const char *pathname, hc_stat_t *buf)
-{
-  return stat64 (pathname, buf);
-}
-
-int hc_fstat (int fd, hc_stat_t *buf)
-{
-  return fstat64 (fd, buf);
-}
-#else
-int hc_stat (const char *pathname, hc_stat_t *buf)
-{
-  return stat (pathname, buf);
-}
-
-int hc_fstat (int fd, hc_stat_t *buf)
-{
-  return fstat (fd, buf);
-}
-#endif
-
 #if defined (_WIN)
 #define __WINDOWS__
 #endif
@@ -194,7 +176,7 @@ void *hc_bsearch_r (const void *key, const void *base, size_t nmemb, size_t size
 
     const size_t c = l + m;
 
-    const char *next = (char *) base + (c * size);
+    const char *next = (const char *) base + (c * size);
 
     const int cmp = (*compar) (key, next, arg);
 
@@ -213,9 +195,9 @@ void *hc_bsearch_r (const void *key, const void *base, size_t nmemb, size_t size
 
 bool hc_path_is_file (const char *path)
 {
-  hc_stat_t s;
+  struct stat s;
 
-  if (hc_stat (path, &s) == -1) return false;
+  if (stat (path, &s) == -1) return false;
 
   if (S_ISREG (s.st_mode)) return true;
 
@@ -224,9 +206,9 @@ bool hc_path_is_file (const char *path)
 
 bool hc_path_is_directory (const char *path)
 {
-  hc_stat_t s;
+  struct stat s;
 
-  if (hc_stat (path, &s) == -1) return false;
+  if (stat (path, &s) == -1) return false;
 
   if (S_ISDIR (s.st_mode)) return true;
 
@@ -235,9 +217,9 @@ bool hc_path_is_directory (const char *path)
 
 bool hc_path_is_empty (const char *path)
 {
-  hc_stat_t s;
+  struct stat s;
 
-  if (hc_stat (path, &s) == -1) return false;
+  if (stat (path, &s) == -1) return false;
 
   if (s.st_size == 0) return true;
 
@@ -326,6 +308,10 @@ void setup_environment_variables ()
 
   if (getenv ("POCL_KERNEL_CACHE") == NULL)
     putenv ((char *) "POCL_KERNEL_CACHE=0");
+
+  #if defined (__CYGWIN__)
+  cygwin_internal (CW_SYNC_WINENV);
+  #endif
 }
 
 void setup_umask ()
@@ -341,11 +327,9 @@ void setup_seeding (const bool rp_gen_seed_chgd, const u32 rp_gen_seed)
   }
   else
   {
-    hc_time_t ts;
+    const time_t ts = time (NULL); // don't tell me that this is an insecure seed
 
-    hc_time (&ts);
-
-    srand (ts);
+    srand ((unsigned int) ts);
   }
 }
 
@@ -426,49 +410,12 @@ void hc_fwrite (const void *ptr, size_t size, size_t nmemb, FILE *stream)
   if (rc == 0) rc = 0;
 }
 
-
-hc_time_t hc_time (hc_time_t *t)
-{
-  #if defined (_WIN)
-  return _time64 (t);
-  #else
-  return time (t);
-  #endif
-}
-
-struct tm *hc_gmtime (const hc_time_t *t, MAYBE_UNUSED struct tm *result)
-{
-  #if defined (_WIN)
-  return _gmtime64 (t);
-  #else
-  return gmtime_r (t, result);
-  #endif
-}
-
-char *hc_ctime (const hc_time_t *t, char *buf, MAYBE_UNUSED const size_t buf_size)
-{
-  char *etc = NULL;
-
-  #if defined (_WIN)
-  etc = _ctime64 (t);
-
-  if (etc != NULL)
-  {
-    snprintf (buf, buf_size, "%s", etc);
-  }
-  #else
-  etc = ctime_r (t, buf); // buf should have room for at least 26 bytes
-  #endif
-
-  return etc;
-}
-
 bool hc_same_files (char *file1, char *file2)
 {
   if ((file1 != NULL) && (file2 != NULL))
   {
-    hc_stat_t tmpstat_file1;
-    hc_stat_t tmpstat_file2;
+    struct stat tmpstat_file1;
+    struct stat tmpstat_file2;
 
     int do_check = 0;
 
@@ -478,7 +425,7 @@ bool hc_same_files (char *file1, char *file2)
 
     if (fp)
     {
-      if (hc_fstat (fileno (fp), &tmpstat_file1))
+      if (fstat (fileno (fp), &tmpstat_file1))
       {
         fclose (fp);
 
@@ -494,7 +441,7 @@ bool hc_same_files (char *file1, char *file2)
 
     if (fp)
     {
-      if (hc_fstat (fileno (fp), &tmpstat_file2))
+      if (fstat (fileno (fp), &tmpstat_file2))
       {
         fclose (fp);
 
@@ -540,7 +487,7 @@ bool hc_same_files (char *file1, char *file2)
       tmpstat_file2.st_blocks   = 0;
       #endif
 
-      if (memcmp (&tmpstat_file1, &tmpstat_file2, sizeof (hc_stat_t)) == 0)
+      if (memcmp (&tmpstat_file1, &tmpstat_file2, sizeof (struct stat)) == 0)
       {
         return true;
       }
@@ -549,3 +496,64 @@ bool hc_same_files (char *file1, char *file2)
 
   return false;
 }
+
+u32 hc_strtoul (const char *nptr, char **endptr, int base)
+{
+  return (u32) strtoul (nptr, endptr, base);
+}
+
+u64 hc_strtoull (const char *nptr, char **endptr, int base)
+{
+  return (u64) strtoull (nptr, endptr, base);
+}
+
+u32 power_of_two_ceil_32 (const u32 v)
+{
+  u32 r = v;
+
+  r--;
+
+  r |= r >> 1;
+  r |= r >> 2;
+  r |= r >> 4;
+  r |= r >> 8;
+  r |= r >> 16;
+
+  r++;
+
+  return r;
+}
+
+u32 power_of_two_floor_32 (const u32 v)
+{
+  u32 r = power_of_two_ceil_32 (v);
+
+  if (r > v)
+  {
+    r >>= 1;
+  }
+
+  return r;
+}
+
+u32 round_up_multiple_32 (const u32 v, const u32 m)
+{
+  if (m == 0) return v;
+
+  const u32 r = v % m;
+
+  if (r == 0) return v;
+
+  return v + m - r;
+}
+
+u64 round_up_multiple_64 (const u64 v, const u64 m)
+{
+  if (m == 0) return v;
+
+  const u64 r = v % m;
+
+  if (r == 0) return v;
+
+  return v + m - r;
+}
